import random

class Graph:
    def __init__(self, n = 0, type = "empty", p = .5):
        """Create an empty graph with n vertices"""
        self.vertices = {}
        self.edgeCtr = 0
        for i in range(n):
            self.addVertex(i)
        if type == "empty":
            return
        elif type == "complete" or type == "random":
            for i in range(n):
                for j in range(i + 1, n):
                    if type == "complete" or random.random() <= p:
                        self.addEdge(i, j)
        elif type == "path" or type == "cycle":
            for i in range(n - 1):
                self.addEdge(i, i + 1)
            if type == "cycle" and n > 0:
                self.addEdge(0, n - 1)
        else:
            raise ValueError("Invalid graph type: " + str(type))

    def numVertices(self):
        """Return number of vertices in the graph"""
        return len(self.vertices)

    def numEdges(self):
        """Return number of vertices in the graph"""
        return self.edgeCtr

    def addVertex(self, v):
        """Add vertex v.  Raises an exception if v is already in the graph"""
        if v in self.vertices:
            raise KeyError(self.errorMsg(v, True))
        else:
            self.vertices[v] = {}

    def removeVertex(self, v):
        """Remove vertex v.  Does nothing if v is not in the graph"""
        if v in self.vertices:
            # Remove all edges incident to v
            for u in self.vertices:
                self.removeEdge(u, v)
            self.vertices.pop(v)

    def neighbors(self, v):
        """Return list of neighbors of vertex v"""
        if v not in self.vertices:
            raise KeyError(self.errorMsg(v))
        else:
            return list(self.vertices[v].keys())

    def degree(self, v):
        """Degree of vertex v (counting loops twice)"""
        if v not in self.vertices:
            raise KeyError(self.errorMsg(v))
        else:
            deg = len(self.vertices[v])
            if self.hasEdge(v, v):
                deg += 1
            return deg

    def hasEdge(self, u, v):
        """Return True iff. uv is an edge"""
        if u not in self.vertices:
            raise KeyError(self.errorMsg(u))
        elif v not in self.vertices:
            raise KeyError(self.errorMsg(v))
        else:
            return u in self.vertices[v]

    def addEdge(self, u, v):
        """Add uv to edges.  Does nothing if uv is already an edge."""
        if not self.hasEdge(u, v):
            self.vertices[u][v] = None
            self.vertices[v][u] = None
            self.edgeCtr += 1

    def removeEdge(self, u, v):
        """Remove uv from edges.  Does nothing if uv is not an edge."""
        if self.hasEdge(u, v):
            self.vertices[u].pop(v)
            self.vertices[v].pop(u)
            self.edgeCtr -= 1

    def __str__(self):
        """Return a string containing each vertex's list of neighbors"""
        s = ""
        for v in self.vertices:
            s += str(v) + ": " + str(self.neighbors(v)) + "\n"
        return s

    def errorMsg(self, v, alreadyIn = False):
        if alreadyIn:
            return "Graph already contains " + str(v) + " as a vertex."
        else:
            return "Graph does not contain " + str(v) + " as a vertex."

class WeightedGraph(Graph):
    def addEdge(self, u, v, weight = 1):
        """Add an edge or update its weight"""
        Graph.addEdge(self, u, v)
        self.vertices[u][v] = weight
        self.vertices[v][u] = weight

    def weight(self, u, v):
        """Return the weight of edge uv or None if it's not an edge"""
        if self.hasEdge(u, v):
            return self.vertices[u][v]
        else:
            return None

    def __str__(self):
        s = ""
        for v in self.vertices:
            s += str(v) + ": "
            for u in self.neighbors(v):
                s += str(u) + "(" + str(self.weight(u, v)) + ") "
            s += "\n"
        return s

def EmptyGraph(n):
    return Graph(n)

def CompleteGraph(n):
    return Graph(n, "complete")

def Path(n):
    return Graph(n, "path")

def Cycle(n):
    return Graph(n, "cycle")

def RandomGraph(n, p = .5):
    return Graph(n, "random", p)

def BFS(G, s):
    """BFS in G starting from s"""
    
    n = G.numVertices()
    
    dists = {v: "Infinity" for v in G.vertices}
    
    dists[s] = 0
    toVisit = [s]

    i = 0
    while(i < len(toVisit)):
        v = toVisit[i] # We already know v's distance from s

        # Set distances of all v's neighbors
        for u in G.neighbors(v):
            if dists[u] == "Infinity":
                dists[u] = dists[v] + 1
                toVisit.append(u)

        i += 1

    return dists
